----------------------------------------------------------------------------------
-- Company: 
-- Engineer:       John-Eric Kamps
-- 
-- Create Date:    12:00:00 18/10/2010 
-- Design Name: 
-- Module Name:    Display - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
library UNISIM;
use UNISIM.VComponents.all;

entity Display is
    Port ( clk_in			: in   STD_LOGIC;
	        RST				: in	 STD_LOGIC;
           taster1		: in	 STD_LOGIC;
           taster2		: in	 STD_LOGIC;
           taster3		: in	 STD_LOGIC;
           taster4		: in	 STD_LOGIC;
--			  Strobe			: out  STD_LOGIC;
--			  PowerDown		: out  STD_LOGIC;
           red 			: out  STD_LOGIC_VECTOR (5 downto 0);
           blue 			: out  STD_LOGIC_VECTOR (5 downto 0);
           green 			: out  STD_LOGIC_VECTOR (5 downto 0);
           hsync 			: out  STD_LOGIC:='1';
           vsync 			: out  STD_LOGIC:='1';
           enable 		: out  STD_LOGIC:='0');
end Display;

architecture Behavioral of Display is
----------------------------------------------------------------------------------
--               Displayeinstellungen (Auflsung fr 800x600)                   --
--                            Takt max. 42MHz                                   --
----------------------------------------------------------------------------------
--constant TVC : integer :=4;				-- Sync pulse width
--constant TVD : integer :=23;				-- Back porch
--constant TVE : integer :=600;				-- Active display area
--constant TVF : integer :=39;				-- Front porch
--constant TVB : integer :=TVC+TVD+TVF;	-- Blanking period
--constant TVA : integer :=TVE+TVB;		-- Vsync cycle

--constant THC : integer :=128;				-- Sync pulse width
--constant THD : integer :=88;				-- Back porch
--constant THE : integer :=800;				-- Active display area
--constant THF : integer :=40;				-- Front porch
--constant THB : integer :=THC+THD+THF;	-- Blanking period
--constant THA : integer :=THE+THB;		-- Hsync cycle
----------------------------------------------------------------------------------

----------------------------------------------------------------------------------
--               Displayeinstellungen (Auflsung fr 1024x768)                  --
--                            Takt max. 65MHz                                   --
----------------------------------------------------------------------------------
constant TVC : integer :=6;				-- Sync pulse width
constant TVD : integer :=29;				-- Back porch
constant TVE : integer :=768;				-- Active display area
constant TVF : integer :=3;				-- Front porch
constant TVB : integer :=TVC+TVD+TVF;	-- Blanking period
constant TVA : integer :=TVE+TVB;		-- Vsync cycle

constant THC : integer :=136;				-- Sync pulse width
constant THD : integer :=160;				-- Back porch
constant THE : integer :=1024;			-- Active display area
constant THF : integer :=24;				-- Front porch
constant THB : integer :=THC+THD+THF;	-- Blanking period
constant THA : integer :=THE+THB;		-- Hsync cycle
----------------------------------------------------------------------------------


--Signal RST					: STD_LOGIC:='0';
--Signal clk					: STD_LOGIC:='0';
Signal hcount				: integer range 1 to THA:=THA;

Signal vcount				: integer range 1 to TVA:=TVA;

--Signal test				   : STD_LOGIC_VECTOR(9 downto 0):="0000000000";      

Signal taster1p			: STD_LOGIC_VECTOR(3 downto 0);
Signal taster2p			: STD_LOGIC_VECTOR(3 downto 0);
Signal taster3p			: STD_LOGIC_VECTOR(3 downto 0);
Signal taster4p			: STD_LOGIC_VECTOR(3 downto 0);
Signal Farbe_rot			: STD_LOGIC_VECTOR(5 downto 0);
Signal Farbe_gruen		: STD_LOGIC_VECTOR(5 downto 0);
Signal Farbe_blau			: STD_LOGIC_VECTOR(5 downto 0);


begin
--PowerDown	<= '1';
--Strobe		<= '1';


process(clk_in)
begin
if rising_edge(clk_in) then
  taster1p <= taster1p(2 downto 0) & taster1;
  taster2p <= taster2p(2 downto 0) & taster2;
  taster3p <= taster3p(2 downto 0) & taster3;
  taster4p <= taster4p(2 downto 0) & taster4;

  if taster1p = x"7" then
     Farbe_rot		<= "000000";
     Farbe_gruen	<= "000000";
     Farbe_blau	<= "000000";
  elsif taster2p = x"7" then
     Farbe_rot		<= Farbe_rot-1;
  elsif taster3p = x"7" then
     Farbe_gruen	<= Farbe_gruen-1;
  elsif taster4p = x"7" then
     Farbe_blau	<= Farbe_blau-1;
  end if;
  
  
  if RST = '1' then
     hcount <= THA;
     vcount <= TVA;	  
  elsif hcount = THA then		-- Verticaler Zhler
     hcount <= 1;
	  if vcount = TVA then		-- Horizontaler Zhler
	     vcount <= 1;
	  else
	     vcount <= vcount +1;
	  end if;
  else
	  hcount <= hcount + 1;
  end if;

  -- Horizontaler Sync Impuls
  if hcount >= 1 and hcount < (THC+1) then
     hsync	<= '0';
  else
     hsync	<= '1';
  end if;

  -- Vertikaler Sync Impuls
  if vcount >= 1 and vcount < (TVC+1) then						
     vsync <= '0';
  else
     vsync <= '1';
  end if;

  -- Enable Impuls  
  if (vcount > (TVC+TVD-1) and vcount < (TVC+TVD+TVE+1)) 
       and (hcount > (THC+THD-1) and hcount < (THC+THD+THE+1)) then
     enable	<= '1';
  else
     enable	<= '0';
  end if;
  
  

  -- bergabe Testbild(graustufen)
  if hcount=(THC+THD+100) then    --gelb
     red		<= "111111";
     green	<= "111111";
     blue	<= "000000";
  elsif hcount=(THC+THD+200) then --magenta
     red		<= "111111";
     green	<= "000000";
     blue	<= "111111";
  elsif hcount=(THC+THD+300) then --schwarz
     red		<= "000000";
     green	<= "000000";
     blue	<= "000000";
  elsif vcount=(TVC+TVD+100) then --rot
     red		<= "111111";
     green	<= "000000";
     blue	<= "000000";
  elsif vcount=(TVC+TVD+200) then --grn
     red		<= "000000";
     green	<= "111111";
     blue	<= "000000";
  elsif vcount=(TVC+TVD+300) then --blau
     red		<= "000000";
     green	<= "000000";
     blue	<= "111111";
  else
     red		<= Farbe_rot;
     green	<= Farbe_gruen;
     blue	<= Farbe_blau;
  end if;

end if; --clk
end process;

end Behavioral;

